/*
** Copyright (c) 1995,1996
**      Michael Smith and Genesis Software.  All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions
** are met:
** 1. Redistributions of source code must retain the above copyright
**    notice, this list of conditions and the following disclaimer as
**    the first lines of this file unmodified.
** 2. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
** 3. The name of the author may not be used to endorse or promote products
**    derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY Michael Smith and Genesis Software ``AS IS'' 
** AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
** ARE DISCLAIMED. IN NO EVENT SHALL Michael Smith and or Genesis Software BE
** LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
** POSSIBILITY OF SUCH DAMAGE.
**
**      $Id: mdcat.c,v 1.2 1996/10/22 03:30:37 msmith Exp $
*/
/*
** MDCAT.C
**
** Test program to read from/write to a multidrop serial port.
**
*/

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>

#define barf(fmt, args...)	{fprintf(stderr,"barf: " fmt "\n" , ##args);exit(1);}
#define whine(fmt, args...)	fprintf(stderr,"whine: " fmt "\n" , ##args)

void main(int argc, char *argv[])
{
    int		fd,cfd,rv;
    fd_set	rfd;
    u_char	cbuf[500];
    u_short	pbuf[500];
    int		i,j,pfx;
    int		etw;

    if (argc!=2)
	barf("usage is %s <portname>",argv[0]);
    
    if ((fd = open(argv[1],O_RDWR,0)) == -1)
	barf("can't open %s : %s",argv[1],strerror(errno));
    cfd = fileno(stdin);

    for (;;) {
	FD_ZERO(&rfd);
	FD_SET(cfd,&rfd);
	FD_SET(fd,&rfd);

	rv = select(FD_SETSIZE,&rfd,NULL,NULL,NULL);
	if (rv == -1)
	    barf("select failed : %s",strerror(errno));
	etw = 0;

	if (FD_ISSET(cfd,&rfd))	{
	    rv = read(cfd,cbuf,sizeof(cbuf));
	    if (rv < 1) {
		whine("read on stdin returned %d",rv);
		exit(0);
	    }
	    
	    for (i = 0, j = 0, pfx = 0; i < rv; i++) {
		if (cbuf[i] == 0x1b) {		/* address prefix ? */
		    pfx = 1;
		}else{
		    pbuf[j] = (u_short)cbuf[i];
		    if (pfx)
		    {
			pbuf[j] |= 0x100;
			pfx = 0;
		    }
		    j++;
		}
	    }
	    rv = write(fd,pbuf,j*2);
	    if (rv != j*2)
		whine("failed to write %d bytes (wrote %d)",j*2,rv);
	    etw = 1;
	}
	if (FD_ISSET(fd,&rfd)) {
	    rv = read(fd,pbuf,sizeof(pbuf)/2);
	    if (rv < 1)
		barf("read from device returned %d");
		
	    for (i = 0; i < rv/2; i++)
		printf("%s%02x ",(pbuf[i]&0x100)?"\n*":" ",pbuf[i]&0xff);
	    fflush(stdout);
	    etw = 1;
	}
	if (!etw)
	    whine("select returned for no good reason");
    }
}

		
		
	    

